#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'collect-forecast-data.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    library(rhdf5)

    data_dir <- file.path('output')
    locations <- c('sydney', 'brisbane', 'melbourne', 'perth')
    years <- c(2017, 2015)

    save_forecast_data(locations, years, data_dir)

    return(0)
}

##
## Save the forecast credible intervals and observations for each year in each
## of the forecasting locations.
##
save_forecast_data <- function(locations, years, data_dir) {
    df <- NULL
    obs <- NULL

    for (locn in locations) {
        for (year in years) {
            data <- collect_data(locn, year, data_dir)
            df <- rbind(df, data$df)
            obs <- rbind(obs, data$obs)
        }
    }

    write.table(df, 'forecast-cis.ssv', row.names = FALSE)
    write.table(obs, 'forecast-obs.ssv', row.names = FALSE)
}

##
## Collect the forecast data for a single year in a single location.
##
collect_data <- function(locn, year, data_dir) {
    ## The earliest Flutracking surveys that we've used in the
    ## estimation process are week ending July 1 or later.
    ft_start_date <- as.Date(paste0(year, '-07-01'))
    ft_end_date <- as.Date(paste0(year, '-10-20'))

    ## Select forecasts for kappa values 0..4.
    pattern <- paste0('^', locn, '-slope-[01234]\\.0-', year, '-.*\\.hdf5$')
    fs_files <- list.files(path = data_dir, full.names = TRUE,
                           pattern = pattern)

    titles <- c('sydney' = 'Sydney, NSW',
                'brisbane' = 'Brisbane, QLD',
                'melbourne' = 'Melbourne, VIC',
                'perth' = 'Perth, WA')

    df <- NULL
    obs <- NULL

    for (fs_file in fs_files) {
        ## NOTE: need to extract the forecasting date from the filename.
        pattern <- paste0('.*-(', year, '-\\d\\d-\\d\\d)-.*')
        date_str <- sub(pattern, '\\1', fs_file,
                        perl = TRUE)
        fs_date <- as.Date(date_str)

        df_fs <- h5read(fs_file, '/data/forecasts')
        df_fs$fs_date <- fs_date
        df_fs$date <- as.Date(df_fs$date)

        if (fs_date < ft_start_date || fs_date > ft_end_date + 14) {
            next
        }

        ## Identify the value of the slope parameter (kappa).
        slope_str <- sub('.*-slope-(\\d+\\.\\d+)-.*', '\\1', fs_file,
                         perl = TRUE)
        df_fs$slope <- as.numeric(slope_str)

        df_fs <- df_fs[df_fs$date >= df_fs$fs_date, ]

        df <- rbind(df, df_fs)

        df_obs <- h5read(fs_file, '/data/obs/Notifications')
        df_obs$date <- as.Date(df_obs$date)
        df_obs$fs_date <- fs_date
        df_obs$future <- FALSE
        df_obs$future[df_obs$date > fs_date] <- TRUE
        df_obs <- df_obs[df_obs$date >= ft_start_date - 14, ]
        df_obs <- df_obs[df_obs$date <= ft_end_date + 7, ]
        obs <- rbind(obs, df_obs)
    }

    df <- df[df$date <= max(obs$date), ]

    ## Remove columns that are not required.
    df$unit <- NULL
    df$period <- NULL
    obs$source <- NULL
    obs$unit <- NULL
    obs$period <- NULL

    ## Identify the location and year.
    df$locn <- locn
    df$year <- year
    obs$locn <- locn
    obs$year <- year

    list(df = df, obs = obs)
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
