#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'collect-results.R'

##
## The script entry point.
##
main <- function(args = NULL) {
    library(rhdf5)

    if (length(args) == 1 && args[1] == '--fast') {
        data_dir <- 'fast-output'
    }
    else if (length(args) == 1 && args[1] == '--slow') {
        data_dir <- 'output'
    } else {
        ## Default to using the complete simulation results.
        data_dir <- 'output'
    }

    locns <- c('melbourne', 'brisbane', 'sydney', 'perth')
    years <- 2014:2017
    df <- collect_results(locns, years, data_dir = data_dir)
    if (nrow(df) == 0) {
        cat('No data files found, exiting.\n')
        return(0)
    }
    df <- bayes_factors(df, relative_to_slope = 0)
    df_pr <- obs_probability(df)

    ##
    ## Ensure the data are written in a specific order.
    ##
    df <- df[order(df$locn, df$fs_date, df$measure, df$slope), ]
    df_pr <- df_pr[order(df_pr$locn, df_pr$fs_date, df_pr$slope), ]

    ##
    ## Save these results to disk.
    ##
    bf_file <- 'bayes-factors.ssv'
    cat('Writing', bf_file, '...')
    write.table(df, bf_file, row.names = FALSE)
    cat(' done\n')

    pr_file <- 'pr-obs.ssv'
    cat('Writing', pr_file, '...')
    write.table(df_pr, pr_file, row.names = FALSE)
    cat(' done\n')

    return(0)
}


##
## Extract the observation probability for each forecast.
##
obs_probability <- function(df) {
    pr_cols <- c('locn', 'fs_date', 'slope', 'pr_obs', 'Kappa')
    df_pr <- unique(df[, pr_cols])
    return(df_pr)
}


##
## Calculate Bayes factors.
##
bayes_factors <- function(df, relative_to_slope = 0) {
    df_bf <- df[df$slope == relative_to_slope, ]
    df_bf$null_log_llhd <- df_bf$log_llhd
    df_bf$log_llhd <- NULL
    df_bf$slope <- NULL
    df_bf$pr_obs <- NULL
    df <- merge(df, df_bf,
                by = c('locn', 'fs_date', 'measure'))
    df$BayesFactor <- df$log_llhd - df$null_log_llhd
    df$Kappa <- factor(df$slope)
    return(df)
}


##
## Collect the observation likelihoods for each simulation.
##
collect_results <- function(locns, years, data_dir = 'output') {
    out_locn <- c()
    out_slope <- c()
    out_fs_date <- c()
    out_log_llhd_future <- c()
    out_log_llhd_past <- c()
    out_log_llhd_net <- c()
    out_log_llhd_season <- c()
    out_log_llhd_now <- c()
    out_log_llhd_next2 <- c()
    out_log_llhd_next4 <- c()
    out_log_llhd_next6 <- c()
    out_pr_obs <- c()

    for (locn in locns) {
        locn_name <- sub('\\b(\\w)', '\\U\\1', locn, perl=TRUE)
        cat('Reading results for', locn_name, '...')

        for (year in years) {
            ## The earliest Flutracking surveys that we've used in the
            ## estimation process are week ending July 1 or later.
            ft_start_date <- as.Date(paste0(year, '-07-01'))
            ft_end_date <- as.Date(paste0(year, '-10-20'))

            ## Define an appropriate epidemic interval.
            ## NOTE: it may be location-dependent and/or year-dependent.
            season_start <- as.Date(paste0(year, '-07-01'))
            season_end <- as.Date(paste0(year, '-09-30'))

            ## Identify the range of values for the slope parameter.
            locn_pat <- paste0('^', locn, '-slope-\\d+\\.\\d+-',
                               year, '-.*\\.hdf5$')
            locn_files <- list.files(path = data_dir, pattern = locn_pat,
                                     full.names = TRUE)
            slope_vals <- sub('.*-slope-(\\d+\\.\\d+)-.*', '\\1', locn_files,
                              perl=TRUE)
            slope_list <- sort(unique(slope_vals))

            for (slope in slope_list) {

                pat <- paste0('^', locn, '-slope-', slope, '-', year,
                              '-.*\\.hdf5$')
                data_files <- list.files(path = data_dir, pattern = pat,
                                         full.names = TRUE)

                for (data_file in data_files) {
                    ## Extract the forecasting date from the filename
                    fs_date_str <- sub('-bg-.*$', '',
                                       sub(paste0('^.*-', year), year,
                                           data_file))
                    fs_date <- as.Date(fs_date_str)

                    if (fs_date < ft_start_date - 7 || fs_date > ft_end_date) {
                        next
                    }

                    llhd_path <- '/data/obs_llhd'

                    ## NOTE: also record the observation probability.
                    pobs_path <- '/meta/param/obs/Notifications/pr_obs'
                    pr_obs <- h5read(data_file, pobs_path)

                    df <- h5read(data_file, llhd_path)
                    df$fs_date <- fs_date
                    df$date <- as.Date(df$date)

                    ## Restrict likelihoods so that the logs are finite.
                    zero_llhd <- df$llhd == 0
                    if (any(zero_llhd)) {
                        df$llhd[zero_llhd] <- 1e-300
                    }

                    df_future <- df[df$date > df$fs_date, ]
                    df_past <- df[df$date <= df$fs_date, ]
                    df_season <- df[df$date >= season_start
                                    & df$date <= season_end, ]

                    if (nrow(df_future) == 0 || nrow(df_past) == 0) {
                        cat(locn, 'has', nrow(df_past), 'past and',
                            nrow(df_future), 'obs at', fs_date_str, '...\n')
                        next
                    }

                    log_llhd_future <- sum(log(df_future$llhd))
                    log_llhd_past <- sum(log(df_past$llhd))
                    log_llhd_net <- sum(log(df$llhd))
                    log_llhd_season <- sum(log(df_season$llhd))

                    mask_now <- df$date == fs_date
                    mask_next2 <- df_future$date <= fs_date + 2 * 7
                    mask_next4 <- df_future$date <= fs_date + 4 * 7
                    mask_next6 <- df_future$date <= fs_date + 6 * 7
                    log_llhd_now <- sum(log(df$llhd[mask_now]))
                    log_llhd_next2 <- sum(log(df_future$llhd[mask_next2]))
                    log_llhd_next4 <- sum(log(df_future$llhd[mask_next4]))
                    log_llhd_next6 <- sum(log(df_future$llhd[mask_next6]))
                    out_log_llhd_now <- c(out_log_llhd_now, log_llhd_now)
                    out_log_llhd_next2 <- c(out_log_llhd_next2,
                                            log_llhd_next2)
                    out_log_llhd_next4 <- c(out_log_llhd_next4,
                                            log_llhd_next4)
                    out_log_llhd_next6 <- c(out_log_llhd_next6,
                                            log_llhd_next6)

                    out_locn <- c(out_locn, locn_name)
                    out_slope <- c(out_slope, as.numeric(slope))
                    out_fs_date <- c(out_fs_date, fs_date_str)
                    out_log_llhd_future <- c(out_log_llhd_future,
                                             log_llhd_future)
                    out_log_llhd_past <- c(out_log_llhd_past, log_llhd_past)
                    out_log_llhd_net <- c(out_log_llhd_net, log_llhd_net)
                    out_log_llhd_season <- c(out_log_llhd_season,
                                             log_llhd_season)
                    out_pr_obs <- c(out_pr_obs, pr_obs)
                }
            }
        }

        cat(' done\n')
    }

    measures <- c('Past observations',
                  'Future observations',
                  'All observations',
                  'In-season observations',
                  'Now-casts',
                  'Next 2 weeks',
                  'Next 4 weeks',
                  'Next 6 weeks')
    n_logs <- length(measures)

    df <- data.frame(locn = rep(out_locn, n_logs),
                     slope = rep(out_slope, n_logs),
                     fs_date = rep(as.Date(out_fs_date), n_logs),
                     pr_obs = rep(out_pr_obs, n_logs),
                     log_llhd = c(out_log_llhd_past, out_log_llhd_future,
                                  out_log_llhd_net, out_log_llhd_season,
                                  out_log_llhd_now, out_log_llhd_next2,
                                  out_log_llhd_next4, out_log_llhd_next6),
                     measure = rep(measures, each = length(out_locn)))

    return(df)
}

##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
