#!/usr/bin/env python

import argparse
import datetime
import epifx
import epifx.cmd
import epifx.cmd.settings
import epifx.obs
import epifx.summary
import logging
import numpy as np
import os.path
import pypfilt
import pypfilt.summary
import sys


def main(args=None):
    p = get_parser()
    if args is None:
        args = vars(p.parse_args())
    else:
        args = vars(p.parse_args(args))
    logging.basicConfig(level=args['loglevel'])

    # Define scaling factors between the P(Test|ILI) estimates and the
    # observation probability in the forecasts.
    slope_min = 0
    slope_max = 10
    slope_step = 0.25
    slopes = np.arange(slope_min, slope_max + 0.5 * slope_step, slope_step)

    # The forecasting locations.
    locns = ['melbourne', 'sydney', 'brisbane', 'perth']

    # Running forecasts for the 2014-2017 seasons.
    args['year'] = [2014, 2015, 2016, 2017]

    # Run the forecasts in using multiple worker processes.
    iterable = forecast_iter(slopes, locns, args['year'], args['fast'])
    epifx.cmd.run_in_parallel(run_forecasts, iterable, args['spawn'])

    return 0


def get_parser():
    p = argparse.ArgumentParser(add_help=False)

    h = p.add_argument_group("Information")
    h.add_argument(
        '-h', '--help', action='help',
        help='Show this help message')

    og = p.add_argument_group('Output options')
    log = og.add_mutually_exclusive_group()
    log.add_argument(
        '-d', '--debug', action='store_const', dest='loglevel',
        help='Enable debugging output',
        const=logging.DEBUG, default=logging.INFO)
    log.add_argument(
        '-q', '--quiet', action='store_const', dest='loglevel',
        help='Suppress logging output',
        const=logging.WARNING)

    cg = p.add_argument_group('Computation settings')
    cg.add_argument(
        '--spawn', metavar='N', type=int, default=1,
        help='Spawn N separate processes')
    cg.add_argument(
        '--fast', action='store_true',
        help='Collect minimal statistics')

    return p


def forecast_iter(slopes, locns, years, fast):
    state = {'melbourne': 'VIC',
             'sydney': 'NSW',
             'brisbane': 'QLD',
             'perth': 'WA'}
    for year in years:
        # Load the P(Test|ILI) estimates derived from the Flutracking data.
        df_post = load_posteriors(year)

        for slope in slopes:
            for locn in locns:
                df_locn = df_post[df_post['state'] == state[locn]]
                prior_pr = df_locn['prior_pr'][0]
                yield (locn, df_locn, prior_pr, slope, year, fast)


def load_posteriors(year):
    """
    Return the posterior expectation for P(Test|ILI) for each week of the
    influenza season.
    """
    data_file = os.path.join('..', 'flutracking-estimation', 'posteriors.ssv')

    # Read in selected columns from the Flutracking estimation output.
    col_ixs = [8, 7, 0, 19, 17, 18]
    dtype = [('state', 'U5'), ('year', int), ('date', 'O'),
             ('expected', float),
             ('prior_alpha', float), ('prior_beta', float)]
    col_conv = {
        0: lambda s: datetime.datetime.strptime(s, '%Y-%m-%d'),
        8: lambda s: s.replace('"', ''),
    }
    df = np.loadtxt(data_file, encoding='utf-8', skiprows=1, usecols=col_ixs,
                    dtype=dtype, converters=col_conv)

    # Retain only the data for the states of interest.
    df = df[df['year'] == year]
    df = df[np.in1d(df['state'], ['NSW', 'QLD', 'VIC', 'WA'])]

    # Calculate the prior expected value for each state.
    prior = df['prior_alpha'] / (df['prior_alpha'] + df['prior_beta'])

    # Construct a structured array that contains only the required fields.
    dtype_out = [('state', 'U5'), ('year', int), ('date', 'O'),
                 ('prior_pr', float), ('pr', float)]
    d = np.dtype(dtype_out)
    rows = [(df['state'][i], df['year'][i], df['date'][i],
             prior[i], df['expected'][i])
            for i in range(df.size)]
    df = np.array(rows, dtype=d)

    return df


def run_forecasts(locn_id, ft_data, prior_pr, pr_slope, year, fast):
    """
    Run forecasting simulations for a single year in a single location.
    """
    logger = logging.getLogger(__name__)

    # Retrieve location-specific parameters.
    settings = local_settings(locn_id, year)
    popn_size = settings['popn']
    forcing_file = settings['forcing_file']
    params_file = settings['params_file']

    # Construct the simulation parameters.
    params = get_params(popn_size, forcing_file, params_file, fast)

    # Define the simulation period.
    start = datetime.datetime(year, 3, 1)
    until = datetime.datetime(year, 10, 31)

    # Ensure a single value is defined for each observation model parameters.
    oms = list(epifx.cmd.settings.param_generator(settings, year, 'om_params'))
    if len(oms) != 1:
        msg = "Invalid observation model parameters: {}"
        raise ValueError(msg.format(oms))

    observer = settings['obs_model']
    om_params, output_suffix, _ = oms[0]
    base_pr_obs = om_params['pr_obs']

    # Load all of the observations that occur in the simulation period.
    all_obs = [o for o in observer.from_file(settings['obs_file'], year=year,
                                             **settings['from_file_args'])
               if o['date'] > start and o['date'] <= until]

    fs_dates = [o['date'] for o in all_obs]

    if fast:
        fs_dates = [d for d in fs_dates
                    if d >= datetime.datetime(year, 6, 20)
                    and d <= datetime.datetime(year, 10, 20)]

    for fs_date in fs_dates:
        # Check whether adjust the observation probability should be adjusted.
        mask = ft_data['date'] == fs_date
        if np.any(mask) and pr_slope != 0:
            expect_pr_obs = ft_data['pr'][mask][0]
            # NOTE: the scaling is based on the ratio of the current
            # expectation for P(Test|ILI) to the prior expectation.
            ratio = (expect_pr_obs - prior_pr) / prior_pr
            # NOTE: when the slope is large and the current expectation for
            # P(Test|ILI) is smaller than the prior expectation, using linear
            # scaling may result in a negative value for pr_obs.
            # Instead raise the ratio to an exponent such that:
            # 1. the value is strictly positive;
            # 2. the value is zero when the ratio is -1; and
            # 3. the function is smooth.
            if ratio < 0:
                adj = np.power(ratio + 1, pr_slope) - 1
            else:
                adj = pr_slope * ratio
            om_params['pr_obs'] = base_pr_obs * (1 + adj)
        else:
            om_params['pr_obs'] = base_pr_obs

        logger.info("  {}  Forecasting from {} to {}".format(
            datetime.datetime.now().strftime("%H:%M:%S"), fs_date, until))

        # Update the observation model.
        observer.define_params(params, **om_params)

        # Distinguish between past and future observations.
        obs_seen = [o for o in all_obs if o['date'] <= fs_date]
        obs_future = [o for o in all_obs if o['date'] > fs_date]

        meta = epifx.summary.Metadata()
        metadata = meta.build(params)
        summary = pypfilt.summary.HDF5(params, all_obs, meta=metadata)
        summary.add_tables(epifx.summary.ObsLikelihood(extra_obs=obs_future))

        # Add additional summary statistic tables.
        peaks = epifx.summary.PeakMonitor()
        tbls = [
            pypfilt.summary.ModelCIs(),
            pypfilt.summary.ParamCovar(),
            epifx.summary.PrOutbreak(),
            epifx.summary.PredictiveCIs(peaks),
            epifx.summary.PeakSizeAccuracy(peaks),
            epifx.summary.PeakTimeAccuracy(peaks),
            epifx.summary.PeakForecastCIs(peaks),
            epifx.summary.PeakForecastEnsembles(peaks),
        ]
        for obs_unit in params['obs']:
            tbls.append(epifx.summary.Obs(obs_unit))
        if fast:
            tbls = []
        summary.add_tables(*tbls)

        state = pypfilt.run(params, start, until, [obs_seen], summary)

        output_file = "{}-slope-{:0.2f}-{}-{}.hdf5".format(
            locn_id, pr_slope, fs_date.date(), output_suffix)

        logger.info("  {}  Saving to:  {}".format(
            datetime.datetime.now().strftime("%H:%M:%S"), output_file))

        forecasts = {'complete': state}
        output_path = os.path.join(params['out_dir'], output_file)
        summary.save_forecasts(forecasts, output_path)

    return


def local_settings(locn_id, year):
    # Default values for parameters that rarely need to be changed.
    defaults = {
        'from_file_args': {},
        'obs_model': epifx.obs.PopnCounts("Notifications", 7),
        'om_format': {
            'bg_obs': '03.0f',
            'bg_var': '03.0f',
            'pr_obs': '0.5f',
            'disp': '03.0f',
        },
        'om_name': {
            'bg_obs': 'bg',
            'bg_var': 'bgvar',
            'pr_obs': 'pr',
            'disp': 'disp',
        },
    }

    def datafile(name):
        return os.path.join('..', 'notifications', name)

    # The dictionary of location-specific settings.
    settings = {
        'melbourne': {
            'name': 'Melbourne',
            'popn': 4108541,
            'obs_file': datafile('vdh-byweek-counts.ssv'),
            'om_params': {
                'bg_obs': {2014: 36, 2015: 73, 2016: 62, 2017: 79},
                'bg_var': {2014: 152, 2015: 122, 2016: 175, 2017: 486},
                'pr_obs': [0.00275],
                'disp': [100],
            },
        },
        'sydney': {
            'name': 'Sydney',
            'popn': 4921000,
            'obs_file': datafile('sydney-weekly-notifications.ssv'),
            'om_params': {
                'bg_obs': {2014: 38, 2015: 75, 2016: 138, 2017: 139},
                'bg_var': {2014: 84, 2015: 170, 2016: 926, 2017: 3096},
                'pr_obs': [0.0076],
                'disp': [100],
            },
        },
        'brisbane': {
            'name': 'Brisbane',
            'popn': 2308700,
            'obs_file': datafile('qld-brisbane-weekly-notifications.ssv'),
            'om_params': {
                'bg_obs': {2014: 28, 2015: 48, 2016: 46, 2017: 40},
                'bg_var': {2014: 81, 2015: 159, 2016: 125, 2017: 114},
                'pr_obs': [0.003],
                'disp': [100],
            },
        },
        'perth': {
            'name': 'Perth',
            'popn': 1900000,
            'obs_file': datafile('wa-pathwest-counts.ssv'),
            'from_file_args': {'date_col': 'week'},
            'om_params': {
                'bg_obs': {2014: 10, 2015: 20, 2016: 14, 2017: 5},
                'bg_var': {2014: 18, 2015: 36, 2016: 80, 2017: 10},
                'pr_obs': [0.0020],
                'disp': [100],
            },
        },
    }

    # Finalise the settings for each location.
    for locn in settings:
        # Apply default settings to each location, as needed.
        for k in defaults:
            if k not in settings[locn]:
                settings[locn][k] = defaults[k]

        # Define the file containing the seasonal forcing signal.
        prefix = locn
        forcing_file = os.path.join(
            "data", "seasonal-forcing-{}-{}.ssv".format(year, prefix))
        if os.path.isfile(forcing_file):
            settings[locn]['forcing_file'] = forcing_file
        else:
            print('No forcing file found for {}'.format(settings[locn]['name']))

        # Define the file containing the sampled prior.
        params_file = os.path.join(
            "data", "samples-{}.ssv".format(prefix))
        if os.path.isfile(params_file):
            settings[locn]['params_file'] = params_file
        else:
            print('No samples file found for {}'.format(settings[locn]['name']))

        # Include the location identifier in the settings.
        settings[locn]['id'] = locn

    if locn_id is None:
        return(settings.keys())
    elif locn_id in settings:
        return(settings[locn_id])
    else:
        raise ValueError("Invalid forecasting location '{}'".format(locn_id))


def get_params(popn_size, forcing_file, params_file, fast):
    px_count = 15000
    prng_seed = 3001
    model = epifx.SEIR()
    time = pypfilt.Datetime()

    params = epifx.default_params(px_count, model, time, popn_size, prng_seed)

    params['resample']['threshold'] = 0.25
    params['resample']['regularisation'] = True
    params['epifx']['stoch'] = False
    params['epifx']['forcing'] = epifx.daily_forcing(forcing_file)
    model.load_params(params_file)

    # Restrict the R0 range to the interval [1.0, 1.6].
    params['param_min'][4] = 1.0
    params['param_max'][4] = 1.6
    # The inverse of the incubation period (default values).
    params['param_min'][5] = 1/3
    params['param_max'][5] = 2.0
    # The inverse of the infectious period (default values).
    params['param_min'][6] = 1/3
    params['param_max'][6] = 1.0
    # Keep eta fixed at 1 (i.e., enforce homogeneous mixing).
    params['param_min'][7] = 1.0
    params['param_max'][7] = 1.0
    # Restrict the seasonal forcing.
    params['param_min'][8] = -0.2
    params['param_max'][8] = 0.0
    # Introduce the first infection in the first 7 weeks (default values).
    params['param_min'][9] = 0
    params['param_max'][9] = 50
    # Update the priors.
    params['prior'] = model.priors(params)

    # Set the output and temporary directories.
    if fast:
        out_dir = 'fast-output'
    else:
        out_dir = 'output'
    if not os.path.isdir(out_dir):
        # Create the directory (and missing parents) with mode -rwxr-x---.
        try:
            os.makedirs(out_dir, mode=0o750)
        except OSError as e:
            # Potential race condition with multiple script instances.
            if e.errno != errno.EEXIST:
                print("Warning: could not create {}".format(out_dir))
                print(e)

    params['out_dir'] = out_dir
    params['tmp_dir'] = out_dir

    return params


if __name__ == "__main__":
    sys.exit(main())
