#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'evolve-posteriors.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    ## Define state-specific prior expectations.
    prior_mean <- c('NSW' = 0.029, 'QLD' = 0.028,
                    'VIC' = 0.016, 'WA' = 0.022)
    ## Use a common prior variance for all states.
    prior_varn <- 1e-4

    ## Load Flutracking data for the 2011-2017 seasons.
    df_ft <- load_ft_data()
    ## Only retain states for which we have notifications data.
    df_ft <- df_ft[df_ft$StateName %in% names(prior_mean), ]

    ## Define the upper-bound for evaluating P(Test|ILI) posterior densities.
    pr_max <- 0.12
    n_xs <- 1201

    ## Select columns that contain numbers of ILI cases and influenza tests.
    case_col <- 'FeverCoughN'
    test_col <- 'Tested_weektestedN'

    ## To avoid conditioning on data from the pre-epidemic interval, only
    ## consider Flutracking surveys from 1 July or later.
    min_survey_date <- '-07-01'

    df_exp <- NULL
    df_state_priors <- data.frame(
        StateName = names(prior_mean),
        Expected = as.numeric(prior_mean))

    ## Evolve a Beta posterior over the surveys for each year in each state.
    for (state in unique(df_ft$StateName)) {
        prior <- make_prior(prior_mean[state], prior_varn)
        df_state <- df_ft[df_ft$StateName == state, ]
        for (year in unique(df_state$Year)) {
            ## Extract the season-specific Flutracking and surveillance data.
            df_season <- df_state[df_state$Year == year, ]

            ## Don't condition on data from the pre-epidemic interval.
            min_date <- as.Date(paste0(year, min_survey_date))
            df_filter <- df_season[df_season$SurveyWeek >= as.Date(min_date), ]

            ## Evolve a Beta posterior and evaluate the posterior densities.
            df_post <- evolve_posterior(df_filter, prior, case_col, test_col)

            ## Calculate the expected value at each week and the total number
            ## of influenza tests reported across all of the surveys.
            df_post$Expected <- df_post$alpha / (df_post$alpha + df_post$beta)
            df_post$TotalTests <-  sum(df_season[[test_col]])

            df_exp <- rbind(df_exp, df_post)
        }
    }

    ## Calculate 95% CIs.
    df_exp$LowerCI <- qbeta(0.025, df_exp$alpha, df_exp$beta)
    df_exp$UpperCI <- qbeta(0.975, df_exp$alpha, df_exp$beta)

    write.table(df_exp, 'posteriors.ssv', row.names = FALSE)

    return(0)
}


##
## Load Flutracking weekly survey data.
##
load_ft_data <- function(data_dir = 'data') {
    input_file <- file.path(data_dir, 'flutracking-survey-data.ssv')
    df <- read.table(input_file, header = TRUE, as.is = TRUE)

    ## Convert from strings to dates.
    df$SurveyWeek <- as.Date(df$SurveyWeek)
    ## Add a date column where each calendar year overlaps.
    df$OverlapDate <- as.Date(format(df$SurveyWeek, '2016-%m-%d'))

    return(df)
}


##
## Calculate the Beta distribution shape parameters to obtain a given mean and
## variance.
##
make_prior <- function(mean, varn) {
    v <- mean * (1 - mean) / varn - 1
    alpha <- mean * v
    beta <- (1 - mean) * v

    return(list(alpha = alpha, beta = beta))
}


##
## Given a Beta prior, update the Beta shape parameters in response to each
## week of Flutracking surveillance data.
##
evolve_posterior <- function(df_season, prior, case_col, test_col) {
    ## Extract only the relevant columns.
    cols <- c('SurveyWeek', case_col, test_col)
    df_sub <- df_season[, cols]

    ## Calculate the cumulative number of ILI cases and influenza tests.
    df_sub$cum_cases <- ave(df_sub[[case_col]], FUN = cumsum)
    df_sub$cum_tests <- ave(df_sub[[test_col]], FUN = cumsum)
    df_sub$cum_untested <- df_sub$cum_cases - df_sub$cum_tests

    ## Calculate the posterior distribution at each week.
    ##
    ## If we then assume that the observed number of tests follows a binomial
    ## distribution with probability P_obs (say there were $k$ test for $n$
    ## people reporting ILI symptoms) the posterior is a Beta distribution:
    ##
    ##     $Beta(\alpha + k, \beta + n - k)$
    ##
    ##     n: disease incidence in the surveyed individuals; and
    ##     k: the number of surveyed individuals who were tested.
    df_sub$alpha <- ave(df_sub$cum_tests, FUN = function(x) prior$alpha + x)
    df_sub$beta <- ave(df_sub$cum_untested, FUN = function(x) prior$beta + x)

    ## Combine the posterior parameters with the original data frame.
    keep_cols <- c('SurveyWeek', 'cum_cases', 'cum_tests', 'cum_untested',
                   'alpha', 'beta')
    df_post <- merge(df_sub[, keep_cols], df_season)

    ## Insert the posterior as the first row.
    prior_survey <- min(df_post$SurveyWeek) - 7
    df_prior <- data.frame(
        SurveyWeek = prior_survey,
        OverlapDate = as.Date(format(prior_survey, '2016-%m-%d')),
        cum_cases = 0,
        cum_tests = 0,
        cum_untested = 0,
        alpha = prior$alpha,
        beta = prior$beta,
        Year = df_post$Year[1],
        StateName = df_post$StateName[1])

    set_to_zero <- names(df_post)[! names(df_post) %in% names(df_prior)]
    for (name in set_to_zero) {
        df_prior[[name]] <- 0
    }

    df_prior$Prior <- TRUE
    df_post$Prior <- FALSE

    df_post <- rbind(df_prior, df_post)

    ## Record the prior used to generate these posteriors.
    df_post$prior_alpha <- prior$alpha
    df_post$prior_beta <- prior$beta

    return(df_post)
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
