#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'plot-figure-2.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    library(ggplot2)
    library(scales)
    library(Cairo)
    library(viridisLite)
    ## library(themergm)
    source('theme_rgm.R')

    data_file <- 'flutracking-estimation/posteriors.ssv'
    plot_file <- 'figure-2.pdf'

    ## Plot dimensions (in inches).
    plot_w <- 8
    plot_h <- 5.5

    keep_states <- c('NSW', 'QLD', 'VIC', 'WA')
    keep_years <- 2014:2017

    df <- load_epi_data()
    df <- df[df$StateName %in% keep_states & df$Year %in% keep_years, ]

    df$OverlapDate = as.Date(format(df$SurveyWeek, '2016-%m-%d'))
    ## NOTE: data are ordered alphabetically by state, which is how we want to
    ## order the Location factor.
    df$Location <- paste0(df$CityName, ', ', df$StateName)
    df$Location <- factor(df$Location, levels = unique(df$Location),
                          ordered = TRUE)
    df <- df[df$OverlapDate >= as.Date('2016-04-01')
             & df$OverlapDate <= as.Date('2016-12-01'), ]

    p <- ggplot(df, aes(OverlapDate, Cases, colour = factor(Year))) +
        geom_line(size = 1) +
        scale_colour_brewer('Year', palette = 'Dark2') +
        scale_x_date(breaks = date_breaks('2 months'),
                     labels = date_format('%b %-d')) +
        xlab(NULL) +
        ylab('Influenza cases per week') +
        facet_wrap(~ Location, scale = 'free') +
        guides(colour = guide_legend(nrow = 1),
               fill = guide_legend(nrow = 1)) +
        theme_rgm(base.size = 12, key.box = 1.0) +
        theme(legend.text = element_text(vjust = 0.5, hjust = 0,
                                         margin = margin(l = 4, r = 12)),
              legend.position = 'top',
              legend.justification = c(0.5, 0),
              legend.spacing.x = unit(0.25, 'char'))

    cat('Writing', plot_file, '...')
    CairoPDF(plot_file, bg = 'transparent', width = plot_w, height = plot_h)
    print(p)
    invisible(dev.off())
    cat(' done\n')

    return(0)
}


##
## Load surveillance data for each jurisdiction.
##
load_epi_data <- function(data_dir = 'surveillance-data') {
    ## Return a table with columns:
    ##   StateName, CityName, Year, SurveyWeek, Cases.
    cols <- c('StateName', 'CityName', 'Year', 'SurveyWeek', 'Cases')
    nsw <- read.table(
        file.path(data_dir, 'sydney-weekly-notifications.ssv'),
        header = TRUE, as.is = TRUE)
    qld <- read.table(
        file.path(data_dir, 'qld-brisbane-weekly-notifications.ssv'),
        header = TRUE, as.is = TRUE)
    vic <- read.table(
        file.path(data_dir, 'vdh-byweek-counts.ssv'),
        header = TRUE, as.is = TRUE)
    wa  <- read.table(
        file.path(data_dir, 'wa-pathwest-counts.ssv'),
        header = TRUE, as.is = TRUE)
    nsw$StateName <- 'NSW'
    nsw$CityName <- 'Sydney'
    nsw$Year <- nsw$year
    nsw$SurveyWeek <- as.Date(nsw$to)
    nsw$Cases <- nsw$count
    qld$StateName <- 'QLD'
    qld$CityName <- 'Brisbane'
    qld$Year <- qld$year
    qld$SurveyWeek <- as.Date(qld$to)
    qld$Cases <- qld$count
    vic$StateName <- 'VIC'
    vic$CityName <- 'Melbourne'
    vic$Year <- vic$year
    vic$SurveyWeek <- as.Date(vic$to)
    vic$Cases <- vic$count
    wa$StateName <- 'WA'
    wa$CityName <- 'Perth'
    wa$Year <- wa$year
    wa$SurveyWeek <- as.Date(wa$week)
    wa$Cases <- wa$count
    rbind(nsw[, cols],
          qld[, cols],
          vic[, cols],
          wa[, cols])
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
