#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'plot-figure-3a.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    library(ggplot2)
    library(scales)
    library(Cairo)
    ## library(themergm)
    source('theme_rgm.R')

    ## Plot file and dimensions (in inches).
    plot_file <- 'figure-3a.pdf'
    plot_w <- 6
    plot_h <- 4

    df <- build_table()
    p <- plot_table(df)

    cat('Writing', plot_file, '...')
    CairoPDF(plot_file, width = plot_w, height = plot_h)
    print(p)
    invisible(dev.off())
    cat(' done\n')

    return(0)
}


##
## Plot a line for each value of kappa.
##
plot_table <- function(df) {
    ggplot(df, aes(x + 1, p_id, colour = factor(k))) +
        geom_line() +
        xlab(parse(text = 'P(Test ~ "|" ~ ILI, ~ week == w)')) +
        ylab(parse(text = '"Testing probability" ~ p[id](w)')) +
        scale_x_continuous(breaks = c(0, 0.5, 1, 1.5, 2),
                           labels = label_ptests) +
        scale_y_continuous(breaks = pretty_breaks(),
                           labels = label_pids) +
        scale_colour_brewer(NULL,
                            palette = 'Set2',
                            labels = label_kappas) +
        theme_rgm(base.size = 12, key.box = 1,
                  hide.title = FALSE) +
        theme(legend.text = element_text(hjust = 0, vjust = 0.5,
                                         margin = margin(l = 4, r = 12)),
              legend.position = c(0.025, 1),
              legend.justification = c(0, 1))
}


##
## Generate labels for kappa values.
##
label_kappas <- function(x) {
    labels <- c()
    for (i in seq(length(x))) {
        xx <- x[i]
        labels <- c(labels, bquote(kappa == .(xx)))
    }
    labels
}


##
## Generate labels for p_id(w) values.
##
label_pids <- function(x) {
    labels <- c()
    for (i in seq(length(x))) {
        xx <- x[i]
        if (is.na(xx)) {
            new_label <- NA
        } else if (xx == 0) {
            new_label <- expression(0)
        ## } else if (xx == 1) {
        ##     new_label <- expression(p[id](0))
        } else {
            new_label <- bquote(.(xx) %*% p[id](0))
        }
        labels <- c(labels, new_label)
    }
    labels
}


##
## Generate labels of P(Test | ILI) values.
##
label_ptests <- function(x) {
    labels <- c()
    for (i in seq(length(x))) {
        xx <- x[i]
        if (is.na(xx)) {
            new_label <- NA
        } else if (xx == 0) {
            new_label <- expression(0)
        ## } else if (xx == 1) {
        ##     new_label <- expression(mu[0])
        } else {
            new_label <- bquote(.(xx) %*% mu[0])
        }
        labels <- c(labels, new_label)
    }
    labels
}


##
## Tabulate values of the calibration equation, for different values of kappa.
##
build_table <- function(x0 = -1, x1 = 1, by = 1e-3, kappas = 0:3) {
    x <- seq(from = x0, to = x1, by = by)

    xs <- c()
    ys <- c()
    ks <- c()
    for (k in kappas) {
        xs <- c(xs, x)
        ys <- c(ys, f(x, k))
        ks <- c(ks, rep(k, length(x)))
    }

    df <- data.frame(x = xs, y = ys, k = ks)
    df$p_id <- df$y + 1

    df
}


##
## The calibration equation.
##
## f(x) = kappa . x            if x >= 0
##        (x + 1)^kappa - 1    otherwise
##
f <- function(xs, kappa) {
    is_lin <- xs >= 0
    non_lin <- ! is_lin
    ys <- xs
    ys[is_lin] <- kappa * xs[is_lin]
    ys[non_lin] <- (xs[non_lin] +1 ) ^ kappa - 1
    ys
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
