#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'plot-figure-4.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    library(ggplot2)
    library(scales)
    library(Cairo)
    library(viridisLite)
    ## library(themergm)
    source('theme_rgm.R')

    ## Plot file name and dimensions (in inches).
    plot_file <- 'figure-4.pdf'
    plot_w <- 8
    plot_h <- 5.5

    df <- load_data()
    measures <- levels(df$measure)

    ## Calculate the average Bayes factors separately for each season.
    df_ssn_avg <- normalise_bayes_factors(df, 'slope',
                                          c('year', 'locn', 'measure'))

    ## Average the *normalised* Bayes factors for each season prior to 2017.
    df_ssns <- aggregate(cbind(BayesFactor, NormBayesFactor)
                         ~ locn + slope + measure,
                         FUN = mean,
                         data = df_ssn_avg[df_ssn_avg$year < 2017
                                           & df_ssn_avg$slope > 0, ])
    df_prev <- df[df$year < 2017, ]
    df_ssns$year <- paste0(min(df_prev$year), '-', max(df_prev$year))
    max_bf <- max(df_ssns$BayesFactor)

    cat('Writing', plot_file, '...')
    CairoPDF(plot_file, width = plot_w, height = plot_h)
    print(plot_mean_bayes_factors(df_ssns, measures) +
          geom_hline(yintercept = 0, linetype = 'dashed') +
          coord_cartesian(ylim = c(- max_bf, max_bf)))
    invisible(dev.off())
    cat(' done\n')

    return(0)
}


##
## Plot the mean Bayes factors for several measures, over a range of values
## for kappa.
##
plot_mean_bayes_factors <- function(df, measures, hline = FALSE) {
    if (hline) {
        add_h <- geom_hline(yintercept = 0, linetype = 'dashed')
    } else {
        add_h <- geom_blank()
    }
    ggplot(df[df$slope > 0, ], aes(slope, BayesFactor, colour = measure)) +
        add_h +
        geom_line() +
        facet_wrap(~ locn, scale = 'free') +
        xlab('Kappa') +
        ylab(paste('Mean performance', df$year[1])) +
        scale_x_continuous(breaks = pretty_breaks()) +
        scale_y_continuous(breaks = pretty_breaks(4)) +
        scale_colour_brewer('Target', palette = 'Set2') +
        expand_limits(y = 0) +
        theme_rgm(base.size = 12, key.box = 1.0) +
        theme(legend.text = element_text(hjust = 0, vjust = 0.5,
                                         margin = margin(l = 4, r = 12)),
              legend.position = 'top',
              legend.justification = c(0.5, 0),
              legend.spacing.x = unit(0.25, 'char'))
}


##
## Normalise Bayes factors relative to the maximum absolute value achieved
## over some subset of columns.
##
normalise_bayes_factors <- function(df, avg_by, norm_cols) {
    avg_cols <- c(avg_by, norm_cols)
    avg_eqn <- as.formula(paste('BayesFactor ~',
                                paste(avg_cols, collapse = '+')))
    nrm_eqn <- as.formula(paste('AbsBayesFactor ~',
                                paste(norm_cols, collapse = '+')))

    ## Calculate the (absolute) maximum Bayes factor for each group.
    df <- aggregate(avg_eqn, FUN = mean, data = df)
    df$AbsBayesFactor <- abs(df$BayesFactor)
    df_max <- aggregate(nrm_eqn, FUN = max, data = df)
    df_max$MaxBayesFactor <- df_max$AbsBayesFactor
    df_max$AbsBayesFactor <- NULL
    df <- merge(df, df_max)
    df$NormBayesFactor <- 100 * df$BayesFactor / df$MaxBayesFactor
    df$AbsBayesFactor <- NULL
    df$MaxBayesFactor <- NULL
    df
}


##
## Load the per-forecast Bayes factors.
##
load_data <- function(data_dir = 'adjusted-forecasts', measures = NULL) {
    bf_file <- file.path(data_dir, 'bayes-factors.ssv')
    if (is.null(measures)) {
        measures <- c('Now-casts', 'Next 2 weeks',
                      'Next 4 weeks', 'Next 6 weeks')
    }

    df <- read.table(bf_file, header = TRUE, as.is = TRUE)

    ## Identify the calendar year, based on forecasting date.
    df$fs_date <- as.Date(df$fs_date)
    df$year <- as.numeric(format(df$fs_date, '%Y'))
    years <- sort(unique(df$year), decreasing = TRUE)

    ## NOTE: want to order locations alphabetically by state.
    locn_map <- c('Sydney' = 'Sydney, NSW',
                  'Brisbane' = 'Brisbane, QLD',
                  'Melbourne' = 'Melbourne, VIC',
                  'Perth' = 'Perth, WA')
    df$locn <- factor(locn_map[df$locn], ordered = TRUE,
                      levels = locn_map)

    df <- df[df$measure %in% measures, ]
    df$measure <- factor(df$measure, ordered = TRUE, levels = measures)

    df
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
