#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'plot-figure-5.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    library(ggplot2)
    library(scales)
    library(Cairo)
    library(viridisLite)
    ## library(themergm)
    source('theme_rgm.R')

    ## Plot file name and dimensions (in inches).
    plot_file <- 'figure-5.pdf'
    plot_w <- 8
    plot_h <- 5.5

    df <- load_data()
    measures <- levels(df$measure)

    ## Calculate the average Bayes factors separately for each season.
    df_ssn_avg <- normalise_bayes_factors(df, 'slope',
                                          c('year', 'locn', 'measure'))

    ## Average the *normalised* Bayes factors for each season prior to 2017.
    df_ssns <- aggregate(cbind(BayesFactor, NormBayesFactor)
                         ~ locn + slope + measure,
                         FUN = mean,
                         data = df_ssn_avg[df_ssn_avg$year < 2017
                                           & df_ssn_avg$slope > 0, ])

    df_prev <- df[df$year < 2017, ]
    df_ssns$year <- paste0(min(df_prev$year), '-', max(df_prev$year))

    ignore_perth <- c('Perth, WA')

    cat('Writing', plot_file, '...')
    CairoPDF(plot_file, width = plot_w, height = plot_h)
    ## Ignore Perth when plotting the normalised trend.
    print(plot_ribbons(df_ssns, 'NormBayesFactor', ignore_perth) +
          ylab('Mean normalised performance (2014-16)'))
    invisible(dev.off())
    cat(' done\n')

    return(0)
}


##
## Plot averaged Bayes factors for various measures, as well as an overall
## trend, and identify the optimal value of kappa according to this trend.
## Also allow some locations to be excluded from the trend calculation.
##
plot_ribbons <- function(df_lines, ycol, ignore_locns = NULL) {
    ## Calculate the lower and upper bounds for each location.
    df_ribbons <- bayes_factor_limits(df_lines, c('locn', 'slope'))

    ## Calculate overall trends purely as a function of kappa.
    trend_eqn <- as.formula(paste(ycol, '~ slope'))
    if (! is.null(ignore_locns)) {
        df_trend_in <- df_lines[! df_lines$locn %in% ignore_locns, ]
    } else {
        df_trend_in <- df_lines
    }
    df_trend <- aggregate(trend_eqn, FUN = mean, data = df_trend_in)

    best_slope <- df_trend$slope[df_trend[[ycol]] == max(df_trend[[ycol]])]

    ymin <- paste0(ycol, 'Min')
    ymax <- paste0(ycol, 'Max')
    df_lines$y <- df_lines[[ycol]]
    df_ribbons$ymin <- df_ribbons[[ymin]]
    df_ribbons$ymax <- df_ribbons[[ymax]]

    df_trend$y <- df_trend[[ycol]]

    ggplot(NULL, aes(slope, y)) +
        geom_ribbon(data = df_ribbons,
                    mapping = aes(slope, fill = locn, y = NULL,
                                  ymin = ymin, ymax = ymax),
                    show.legend = FALSE,
                    alpha = 0.25) +
        geom_line(data = df_lines,
                  mapping = aes(colour = locn,
                                group = interaction(locn, measure))) +
        geom_hline(yintercept = 0, linetype = 'dashed') +
        geom_line(data = df_trend,
                  colour = 'black',
                  size = 0.75) +
        geom_point(data = NULL,
                   mapping = aes(best_slope, max(df_trend[[ycol]])),
                   size = 3) +
        geom_label(mapping = aes(best_slope, min(df_ribbons$ymin)),
                   colour = 'black',
                   label.size = NA,
                   label = paste('kappa ==', best_slope),
                   parse = TRUE,
                   size = 6,
                   hjust = 0.5,
                   vjust = 0) +
        scale_x_continuous(breaks = pretty_breaks()) +
        scale_y_continuous(breaks = pretty_breaks()) +
        scale_colour_brewer('Location', palette = 'Set2') +
        scale_fill_brewer('Ignore', palette = 'Set2') +
        xlab('Kappa') +
        theme_rgm(base.size = 12, key.box = 1.0) +
        theme(legend.text = element_text(hjust = 0, vjust = 0.5,
                                         margin = margin(l = 4, r = 12)),
              legend.position = 'top',
              legend.justification = c(0.5, 0),
              legend.spacing.x = unit(0.25, 'char'))
}

##
## Calculate the lower and upper bounds of both the Bayes factors and the
## normalised Bayes factors, relative to some subset of columns.
##
bayes_factor_limits <- function(df, group_cols) {
    eqn <- as.formula(paste('cbind(BayesFactor, NormBayesFactor) ~',
                            paste(group_cols, collapse = '+')))
    ## Calculate minimum and maximum values in one step.
    ## https://stackoverflow.com/a/12064297
    df_agg <- aggregate(eqn, data = df,
                        FUN = function(x) c(min = min(x), max = max(x)))
    df_rbn <- do.call(data.frame, df_agg)
    names(df_rbn) <- sub('\\.m', 'M', names(df_rbn))

    df_rbn
}


##
## Normalise Bayes factors relative to the maximum absolute value achieved
## over some subset of columns.
##
normalise_bayes_factors <- function(df, avg_by, norm_cols) {
    avg_cols <- c(avg_by, norm_cols)
    avg_eqn <- as.formula(paste('BayesFactor ~',
                                paste(avg_cols, collapse = '+')))
    nrm_eqn <- as.formula(paste('AbsBayesFactor ~',
                                paste(norm_cols, collapse = '+')))

    ## Calculate the (absolute) maximum Bayes factor for each group.
    df <- aggregate(avg_eqn, FUN = mean, data = df)
    df$AbsBayesFactor <- abs(df$BayesFactor)
    df_max <- aggregate(nrm_eqn, FUN = max, data = df)
    df_max$MaxBayesFactor <- df_max$AbsBayesFactor
    df_max$AbsBayesFactor <- NULL
    df <- merge(df, df_max)
    df$NormBayesFactor <- 100 * df$BayesFactor / df$MaxBayesFactor
    df$AbsBayesFactor <- NULL
    df$MaxBayesFactor <- NULL
    df
}


##
## Load the per-forecast Bayes factors.
##
load_data <- function(data_dir = 'adjusted-forecasts', measures = NULL) {
    bf_file <- file.path(data_dir, 'bayes-factors.ssv')
    if (is.null(measures)) {
        measures <- c('Now-casts', 'Next 2 weeks',
                      'Next 4 weeks', 'Next 6 weeks')
    }

    df <- read.table(bf_file, header = TRUE, as.is = TRUE)

    ## Identify the calendar year, based on forecasting date.
    df$fs_date <- as.Date(df$fs_date)
    df$year <- as.numeric(format(df$fs_date, '%Y'))
    years <- sort(unique(df$year), decreasing = TRUE)

    ## NOTE: want to order locations alphabetically by state.
    locn_map <- c('Sydney' = 'Sydney, NSW',
                  'Brisbane' = 'Brisbane, QLD',
                  'Melbourne' = 'Melbourne, VIC',
                  'Perth' = 'Perth, WA')
    df$locn <- factor(locn_map[df$locn], ordered = TRUE,
                      levels = locn_map)

    df <- df[df$measure %in% measures, ]
    df$measure <- factor(df$measure, ordered = TRUE, levels = measures)

    df
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
