#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'plot-figure-6.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    library(ggplot2)
    library(scales)
    library(Cairo)
    library(viridisLite)
    ## library(themergm)
    source('theme_rgm.R')

    plot_file <- 'figure-6.pdf'
    data_dir <- 'adjusted-forecasts'
    bf_file <- file.path(data_dir, 'bayes-factors.ssv')
    pr_file <- file.path(data_dir, 'pr-obs.ssv')

    ## Plot dimensions (in inches).
    plot_w <- 8
    plot_h <- 5.5

    df <- read.table(bf_file, header = TRUE, as.is = TRUE)
    df_pr <- read.table(pr_file, header = TRUE, as.is = TRUE)

    ## NOTE: discard fractional values of kappa.
    df <- df[df$slope == round(df$slope), ]
    df_pr <- df_pr[df_pr$slope == round(df_pr$slope), ]

    ## Identify the calendar year, based on forecasting date.
    df$fs_date <- as.Date(df$fs_date)
    df_pr$fs_date <- as.Date(df_pr$fs_date)
    df$year <- as.numeric(format(df$fs_date, '%Y'))
    df_pr$year <- as.numeric(format(df_pr$fs_date, '%Y'))
    years <- sort(unique(df$year), decreasing = TRUE)

    ## NOTE: want to order locations alphabetically by state.
    locn_map <- c('Sydney' = 'Sydney, NSW',
                  'Brisbane' = 'Brisbane, QLD',
                  'Melbourne' = 'Melbourne, VIC',
                  'Perth' = 'Perth, WA')
    df$locn <- factor(locn_map[df$locn], ordered = TRUE,
                      levels = locn_map)
    df_pr$locn <- factor(locn_map[df_pr$locn], ordered = TRUE,
                         levels = locn_map)

    df$Kappa <- factor(df$Kappa)
    df_pr$Kappa <- factor(df_pr$Kappa)

    cat('Writing', plot_file, '...')
    CairoPDF(plot_file, width = plot_w, height = plot_h)
    ## Plot results for the 2017 calendar year.
    years <- c(2017)
    for (year in years) {
        print(plot_pr_obs(df_pr[df_pr$year == year, ]))
    }
    invisible(dev.off())
    cat(' done\n')

    return(0)
}


identify_decreased_pr_obs <- function(df_pr, ctrl_slope = 0) {
    ctrl_mask <- df_pr$slope == ctrl_slope
    rest_mask <- ! ctrl_mask

    pr_base <- aggregate(pr_obs ~ locn, data = df_pr[ctrl_mask, ],
                         FUN = unique)
    pr_base$pr_base <- pr_base$pr_obs
    pr_base$pr_obs <- NULL

    df_pr <- merge(df_pr, pr_base)
    df_decr <- df_pr[rest_mask & df_pr$pr_obs < df_pr$pr_base, ]
    df_decr$xmin <- df_decr$fs_date - 4
    df_decr$xmax <- df_decr$fs_date + 4
    df_decr$BayesFactor <- 1
    df_decr$Kappa <- factor(df_decr$slope)

    df_decr
}


plot_pr_obs <- function(df_pr) {
    year <- unique(df_pr$year)
    ggplot(df_pr, aes(fs_date, pr_obs,
                      colour = slope, group = slope)) +
        geom_line() +
        facet_wrap(~ locn, scale = 'free') +
        xlab(NULL) +
        ylab(expression(p[id])) +
        scale_colour_viridis_c("Kappa", breaks = pretty_breaks()) +
        scale_x_date(breaks = date_breaks('1 month'),
                     labels = date_format('%b %-d')) +
        scale_y_continuous(limits = c(0, NA)) +
        guides(colour = guide_colourbar(ticks = FALSE)) +
        theme_rgm(base.size = 12, key.box = 1,
                  hide.title = FALSE) +
        theme(legend.text = element_text(hjust = 0, vjust = 0.5,
                                         margin = margin(l = 4, r = 12)),
              legend.position = 'right',
              legend.justification = c(0, 0.5)) +
        ggtitle(paste0('Observation probability', ' (', year, ')')) +
        theme(plot.title = element_text(hjust = 0.5))
}


##
## Plot the logarithm of the Bayes factors for a single measure.
##
plot_measure <- function(df, measure, df_decr) {
    year <- unique(df$year)
    df_m <- df[df$measure == measure, ]
    ggplot(df_m, aes(fs_date, BayesFactor,
                     colour = slope, group = slope)) +
        ## Shade regions where calibration has decreased pr_obs.
        geom_rect(data = df_decr,
                  mapping = aes(xmin = xmin, xmax = xmax,
                                ymin = - Inf, ymax = Inf),
                  size = 0,
                  colour = '#cfcfcf',
                  fill = '#cfcfcf') +
        geom_line(size = 0.75) +
        facet_wrap(~ locn, scale = 'free') +
        xlab(NULL) +
        ylab('Logarithm of Bayes factor') +
        scale_colour_viridis_c("Kappa", breaks = pretty_breaks()) +
        scale_x_date(breaks = date_breaks('1 month'),
                     labels = date_format('%b %-d')) +
        ggtitle(paste0(measure, ' (', year, ')')) +
        guides(colour = guide_colourbar(ticks = FALSE)) +
        theme_rgm(base.size = 12, key.box = 1,
                  hide.title = FALSE) +
        theme(legend.text = element_text(hjust = 0, vjust = 0.5,
                                         margin = margin(l = 4, r = 12)),
              legend.position = 'right',
              legend.justification = c(0, 0.5),
              plot.title = element_text(hjust = 0.5))
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
