#!/usr/bin/Rscript --vanilla

##
## Used to detect whether this script is being run from the command-line.
##
SCRIPT <- 'plot-figure-8.R'


##
## The script entry point.
##
main <- function(args = NULL) {
    library(ggplot2)
    library(scales)
    library(Cairo)
    ## library(themergm)
    source('theme_rgm.R')

    data <- load_forecast_data()
    df <- data$df
    obs <- data$obs

    ## locations <- c('sydney', 'brisbane', 'melbourne', 'perth')
    ## years <- c(2017, 2015)

    ## Only plot the forecasts shown in Figure 8.
    locations <- c('brisbane')
    years <- c(2017)

    plot_file <- 'figure-8.pdf'
    plot_w <- 16
    plot_h <- 10

    cat('Writing', plot_file, '...')
    CairoPDF(plot_file, width = plot_w, height = plot_h)
    for (year in years) {
        for (locn in locations) {
            plot_df <- df[df$year == year & df$locn == locn, ]
            plot_obs <- obs[obs$year == year & obs$locn == locn, ]
            print(forecast_plot(plot_df, plot_obs))
        }
    }
    invisible(dev.off())
    cat(' done\n')


    return(0)
}


##
## Load the forecast credible intervals and observations.
##
load_forecast_data <- function(data_dir = 'adjusted-forecasts') {
    cis_file <- file.path(data_dir, 'forecast-cis.ssv')
    obs_file <- file.path(data_dir, 'forecast-obs.ssv')

    df <- read.table(cis_file, header = TRUE, as.is = TRUE)
    obs <- read.table(obs_file, header = TRUE, as.is = TRUE)

    df$fs_date <- as.Date(df$fs_date)
    df$date <- as.Date(df$date)
    obs$fs_date <- as.Date(obs$fs_date)
    obs$date <- as.Date(obs$date)

    probs <- sort(unique(df$prob))
    prob_names <- as.character(probs)
    probs <- paste0(probs, '% CI')
    probs[1] <- 'Median'
    names(probs) <- prob_names

    df$ci <- probs[as.character(df$prob)]
    df$ci <- factor(df$ci, ordered = TRUE, levels = rev(probs))

    df$kappa <- paste('Kappa =', df$slope)

    date_fmt <- '%b %-d'
    df$fs_date_lbl <- format(df$fs_date, date_fmt)
    df$fs_date_lbl <- factor(df$fs_date_lbl, ordered = TRUE,
                             levels = format(sort(unique(df$fs_date)),
                                             date_fmt))
    obs$fs_date_lbl <- factor(format(obs$fs_date, date_fmt), ordered = TRUE,
                              levels = levels(df$fs_date_lbl))

    list(df = df, obs = obs)
}


##
## Plot the forecast credible intervals and observations.
##
forecast_plot <- function(df, obs) {
    date_fmt <- '%b %-d'
    titles <- c('sydney' = 'Sydney, NSW',
                'brisbane' = 'Brisbane, QLD',
                'melbourne' = 'Melbourne, VIC',
                'perth' = 'Perth, WA')
    fs_date0 <- min(df$fs_date)

    year <- df$year[1]
    locn <- df$locn[1]

    ## Select only 3 forecasting dates (previously included one earlier
    ## forecast for a total of 4 forecasting dates).
    keep_fs <- fs_date0 + 7 * c(4, 6, 8)

    df_p <- df[df$fs_date %in% keep_fs, ]
    obs_p <- obs[obs$fs_date %in% keep_fs, ]

    ## Define a 'wrap_lbl' factor for faceting plots by a single factor,
    ## allowing the use of facet_wrap() and showing axes on every sub-plot.
    df_p$wrap_lbl <- paste0(df_p$fs_date_lbl, ' (', df_p$kappa, ')')
    wrap_lvls <- c()
    for (slope in sort(unique(df_p$slope))) {
        wrap_lvls <- c(wrap_lvls,
                       paste0(levels(df_p$fs_date_lbl),
                              ' (Kappa = ', slope, ')'))
    }
    df_p$wrap_lbl <- factor(df_p$wrap_lbl, ordered = TRUE,
                            levels = wrap_lvls)
    obs_df <- NULL
    for (kappa in unique(df_p$kappa)) {
        obs_tmp <- obs_p
        obs_tmp$kappa <- kappa
        obs_tmp$wrap_lbl <- paste0(obs_tmp$fs_date_lbl, ' (', kappa, ')')
        obs_df <- rbind(obs_df, obs_tmp)
    }
    obs_p <- obs_df
    obs_p$wrap_lbl <- factor(obs_p$wrap_lbl, ordered = TRUE,
                             levels = wrap_lvls)

    ## Highlight near-optimal forecasts with a different background colour.
    bg_data <- unique(df_p[df_p$slope %in% c(2,3),
                           c('date', 'wrap_lbl')])

    ## Ensure consistent y-axis limits across each row.
    ymax_data <- aggregate(ymax ~ slope, data = df_p, FUN = max)
    ymax_data <- merge(ymax_data,
                       unique(df_p[, c('slope', 'wrap_lbl')]))
    ymax_data$date <- min(df_p$date)

    p <- ggplot(df_p, aes(date)) +
        geom_rect(data = bg_data,
                  fill = '#dfdfdf',
                  xmin = -Inf, xmax = Inf,
                  ymin = -Inf, ymax = Inf) +
        geom_point(data = ymax_data,
                   mapping = aes(y = ymax),
                   alpha = 0.0) +
        geom_ribbon(mapping = aes(ymin = ymin, ymax = ymax, fill = ci)) +
        geom_line(
            data = df_p[df_p$prob == 0, ],
            mapping = aes(y = ymin, colour = ci)) +
        geom_point(data = obs_p,
                   mapping = aes(date, value),
                   size = 2) +
        geom_point(data = obs_p[obs_p$future, ],
                   mapping = aes(date, value),
                   colour = 'white',
                   size = 1.2) +
        scale_colour_brewer(palette = 'Blues', drop = FALSE) +
        scale_fill_brewer(palette = 'Blues') +
        scale_x_date(breaks = date_breaks('1 month'),
                     labels = date_format(date_fmt)) +
        scale_y_continuous(breaks = pretty_breaks(n = 3)) +
        xlab(NULL) +
        ylab("Notified cases per week") +
        ggtitle(paste0(titles[locn], ', ', year)) +
        facet_wrap(~ wrap_lbl, scale = 'free',
                   ncol = length(unique(df_p$fs_date_lbl))) +
        theme_rgm(base.size = 12, key.box = 1.0) +
        theme(legend.text = element_text(hjust = 0, vjust = 0.5,
                                         margin = margin(l = 4, r = 12)),
              legend.position = 'top',
              legend.justification = c(0.5, 0),
              legend.spacing.x = unit(0.25, 'char'),
              plot.title = element_text(hjust = 0.5))

    return(p)
}


##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep('^--file=', commandArgs(FALSE), value = TRUE)
    file.path <- substring(file.arg, 8)
    file.regex <- paste0('^(|.*/)', SCRIPT)
    if (grepl(file.regex, file.path)) {
        quit(status = main(args = commandArgs(TRUE)))
    }
}
